package com.ibm.examples;

/*
 * ** THIS example is provided AS-IS, without warranty.
 * ** Do not contact the IBM or IBM employees for support 
 * ** of this example as no support will be provided.
 *
 * File:
 *	DumpAgent.jar
 *
 * Description:
 *  	This example demonstrates how to create a Java based agent that will
 * 	generate a javacore, heapdump, or process core based on a specific 
 *	threshold.
 *
 * Syntax:
 *	JAVA_HOME/bin/java -javaagent:dumpagent.jar="OPTIONS" MAIN_CLASS
 *
 * 	where OPTIONS can be (all options are required)
 * 	-l
 *		Confirms that you have read and accept the license / user agreement
 *	-d java|heap|system[,java|heap|system]
 *		java = javacore
 *		heap = heapdump
 *		system = process core file
 *	-t theshold_percent 
 *		percent of java heap used (e.g., 80 = Java heap is 80% used)
 *	-r range_start..range_stop
 *		range_start = start range
 *		range_stop = stop range
 *		(e.g., 1..1 = generate 1 dump, 1..4 = generate 4 dumps)
 *
 * History:
 *	2016-08-30:	Prakash Palaniappan (IBM Java Development)	Created examples
 *	2016-09-20:	Roger Leuckie (IBM AIX Java Support)		Updated / Published
 *
 */

import java.lang.instrument.Instrumentation;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryPoolMXBean;
import javax.management.NotificationListener;
import javax.management.NotificationEmitter;
import javax.management.Notification;
import java.lang.management.MemoryType;

class DumpAgent {
    private static boolean init;
    private static int threshold;
    private static boolean acceptedLicense = false;
    public static String [] dumpOptions;
    public static int start = 0;
    public static int end = Integer.MAX_VALUE;

    // DumpAgent command line args: -javaagent:gcDumpAgent.jar="-t 80 -d java,system -r start..end"
    public static void premain(String args, Instrumentation instrumentation) {
        parseCommandLineArgs(args.trim());

        MemoryMXBean mbean = ManagementFactory.getMemoryMXBean();
        NotificationEmitter emitter = (NotificationEmitter) mbean;
        MyListener listener = new MyListener();
        emitter.addNotificationListener(listener, null, null);
        init();
    }

    private static void parseCommandLineArgs(String args) {
        String arguments = args;
        String [] tokens = arguments.split("-");
        for(String token : tokens) {
            if(token.length() > 0) {
                switch(token.charAt(0)) {
                    case 't':
                        threshold = Integer.parseInt(token.substring(2, token.trim().length()));
                        break;
                    case 'd':
                        String dumpOption = token.substring(2, token.length());
                        dumpOptions = dumpOption.trim().split(",");
                        break;
		    case 'l':
			acceptedLicense = true;
                        break;
                    case 'r':
                        String[] rangeStr = token.substring(2).split("\\.\\.");
                        switch(rangeStr.length)
                        {
                            case 2:
                                end = Integer.parseInt(rangeStr[1]);
                            case 1:
                                start = Integer.parseInt(rangeStr[0]);
                                break;
                            default:
                                System.err.println("[ ERROR ] DumpAgent: -r option is invalid");
                                break;
                        }
                    default: // TODO: Nothing to do here. Handle error if required
                        break;
                }
            }
        }
    }

    public synchronized static void init() {
	if ( ! acceptedLicense) {
		System.err.println( "[ WARNING ] DumpAgent: License (-l) not accepted. Restart agent with '-l' option.");
		return;
	}
        else if(( init) && ( acceptedLicense)) {
            return;
	}

        for(MemoryPoolMXBean pool : ManagementFactory.getMemoryPoolMXBeans()) {
            if (pool.getType() == MemoryType.HEAP && pool.isUsageThresholdSupported()) {
                long maxMemory = pool.getUsage().getMax();
                long warningThreshold = (long) (maxMemory * ((double) threshold/100) );
                System.out.println("[ INFO ] DumpAgent: MaxMemory: " + maxMemory + " bytes, WarningThreshold: " + warningThreshold + " bytes");

                pool.setUsageThreshold(warningThreshold);
            }
        }
        init = true;
    }
}

