package com.ibm.examples;

/*
 * ** THIS example is provided AS-IS, without warranty.
 * ** Do not contact the IBM or IBM employees for support
 * ** of this example as no support will be provided.
 *
 * File:
 *      DumpAgent.jar
 *
 * Description:
 *      This example demonstrates how to create a Java based agent that will
 *      generate a javacore, heapdump, or process core based on a specific
 *      threshold.
 *
 * Syntax:
 *      JAVA_HOME/bin/java -javaagent:dumpagent.jar="OPTIONS" MAIN_CLASS
 *
 *      where OPTIONS can be (all options are required)
 *      -l
 *              Confirms that you have read and accept the license / user agreement
 *      -d java|heap|system[,java|heap|system]
 *              java = javacore
 *              heap = heapdump
 *              system = process core file
 *      -t theshold_percent
 *              percent of java heap used (e.g., 80 = Java heap is 80% used)
 *      -r range_start..range_stop
 *              range_start = start range
 *              range_stop = stop range
 *              (e.g., 1..1 = generate 1 dump, 1..4 = generate 4 dumps)
 *
 * History:
 *      2016-08-30:     Prakash Palaniappan (IBM Java Development)      Created examples
 *      2016-09-20:     Roger Leuckie (IBM AIX Java Support)            Updated / Published
 *
 */


import javax.management.NotificationListener;
import javax.management.Notification;
import java.lang.management.MemoryNotificationInfo;
import javax.management.openmbean.CompositeData;

class MyListener implements javax.management.NotificationListener {
    static int eventCnt = 0;

    public synchronized void handleNotification(Notification notification, Object cbData) {
        CompositeData cd = (CompositeData) notification.getUserData();
        MemoryNotificationInfo info = MemoryNotificationInfo.from(cd);

        String notifType = notification.getType();
        if (notifType.equals(MemoryNotificationInfo.MEMORY_THRESHOLD_EXCEEDED)) {
            System.out.println("[ INFO ] DumpAgent: Memory usage reached configured threshold " + info.getUsage());
            eventCnt++;
            if (DumpAgent.start <= eventCnt && DumpAgent.end >= eventCnt)
            {
                for(String agent : DumpAgent.dumpOptions) {
                    if(agent.toLowerCase().contentEquals("system")) {
                        System.out.println("[ INFO ] DumpAgent: Generating SystemDump...");
                        com.ibm.jvm.Dump.SystemDump();
                    } else if(agent.toLowerCase().contentEquals("java")) {
                        System.out.println("[ INFO ] DumpAgent: Generating JavaDump...");
                        com.ibm.jvm.Dump.JavaDump();
                    } else if(agent.toLowerCase().contentEquals("heap")) {
                        System.out.println("[ INFO ] DumpAgent: Generating HeapDump...");
                        com.ibm.jvm.Dump.HeapDump();
                    } else {
                        System.err.println("[ ERROR ] DumpAgent: Invalid Agent configured");
                    }
                }
            }
        }
    }
}

